@@ -89,8 +89,15 @@ Envelope* SOAPClient::receive()
     XercesJanitor<DOMDocument> janitor(doc);
 
     Category& log = Category::getInstance(XMLTOOLING_LOGCAT".SOAPClient");
-    if (log.isDebugEnabled())
+    if (log.isDebugEnabled()) {
+#ifdef XMLTOOLING_LOG4SHIB
         log.debugStream() << "received XML:\n" << *(doc->getDocumentElement()) << logging::eol;
+#else
+        string buf;
+        XMLHelper::serialize(doc->getDocumentElement(), buf);
+        log.debugStream() << "received XML:\n" << buf << logging::eol;
+#endif
+    }
     
     auto_ptr<XMLObject> xmlObject(XMLObjectBuilder::buildOneFromElement(doc->getDocumentElement(), true));
     janitor.release();
